@@ -262,14 +262,10 @@ class AccountController(BaseStorageServer):
         else:
             try:
                 # disallow methods which are not publicly accessible
-                try:
-                    if req.method not in self.allowed_methods:
-                        raise AttributeError('Not allowed method.')
-                except AttributeError:
+                if req.method not in self.allowed_methods:
                     res = HTTPMethodNotAllowed()
                 else:
-                    method = getattr(self, req.method)
-                    res = method(req)
+                    res = getattr(self, req.method)(req)
             except HTTPException as error_response:
                 res = error_response
             except (Exception, Timeout):
