@@ -163,7 +163,7 @@ __all__ = ['TempURL', 'filter_factory',
 
 
 from os.path import basename
-from time import time
+from time import time, strftime, gmtime
 
 from six.moves.urllib.parse import parse_qs
 from six.moves.urllib.parse import urlencode
@@ -400,7 +400,7 @@ class TempURL(object):
 
         def _start_response(status, headers, exc_info=None):
             headers = self._clean_outgoing_headers(headers)
-            if env['REQUEST_METHOD'] == 'GET' and status[0] == '2':
+            if env['REQUEST_METHOD'] in ('GET', 'HEAD') and status[0] == '2':
                 # figure out the right value for content-disposition
                 # 1) use the value from the query string
                 # 2) use the value from the object metadata
@@ -425,6 +425,11 @@ class TempURL(object):
                 # newline into existing_disposition
                 value = disposition_value.replace('\n', '%0A')
                 out_headers.append(('Content-Disposition', value))
+
+                # include Expires header for better cache-control
+                out_headers.append(('Expires', strftime(
+                    "%a, %d %b %Y %H:%M:%S GMT",
+                    gmtime(temp_url_expires))))
                 headers = out_headers
             return start_response(status, headers, exc_info)
 
