@@ -304,10 +304,18 @@ class SegmentedIterable(object):
 
     :param req: original request object
     :param app: WSGI application from which segments will come
+
     :param listing_iter: iterable yielding the object segments to fetch,
-                         along with the byte subranges to fetch, in the
-                         form of a tuple (object-path, first-byte, last-byte)
-                         or (object-path, None, None) to fetch the whole thing.
+        along with the byte subranges to fetch, in the form of a 5-tuple
+        (object-path, object-etag, object-size, first-byte, last-byte).
+
+        If object-etag is None, no MD5 verification will be done.
+
+        If object-size is None, no length verification will be done.
+
+        If first-byte and last-byte are None, then the entire object will be
+        fetched.
+
     :param max_get_time: maximum permitted duration of a GET request (seconds)
     :param logger: logger object
     :param swift_source: value of swift.source in subrequest environ
