@@ -19,6 +19,7 @@ import sys
 import time
 import signal
 import re
+from os.path import basename, dirname, join
 from random import shuffle
 from swift import gettext_ as _
 from contextlib import closing
@@ -28,7 +29,8 @@ from swift.obj import diskfile, replicator
 from swift.common.utils import (
     get_logger, ratelimit_sleep, dump_recon_cache, list_from_csv, listdir,
     unlink_paths_older_than, readconf, config_auto_int_value)
-from swift.common.exceptions import DiskFileQuarantined, DiskFileNotExist
+from swift.common.exceptions import DiskFileQuarantined, DiskFileNotExist,\
+    DiskFileDeleted
 from swift.common.daemon import Daemon
 from swift.common.storage_policy import POLICIES
 
@@ -43,7 +45,6 @@ class AuditorWorker(object):
         self.conf = conf
         self.logger = logger
         self.devices = devices
-        self.diskfile_router = diskfile.DiskFileRouter(conf, self.logger)
         self.max_files_per_second = float(conf.get('files_per_second', 20))
         self.max_bytes_per_second = float(conf.get('bytes_per_second',
                                                    10000000))
@@ -56,17 +57,25 @@ class AuditorWorker(object):
         except (KeyError, SystemExit):
             # if we can't parse the real config (generally a KeyError on
             # __file__, or SystemExit on no object-replicator section) we use
-            # a very conservative default
-            default = 86400
+            # a very conservative default for rsync_timeout
+            default_rsync_timeout = 86400
         else:
             replicator_rsync_timeout = int(replicator_config.get(
                 'rsync_timeout', replicator.DEFAULT_RSYNC_TIMEOUT))
             # Here we can do some light math for ops and use the *replicator's*
             # rsync_timeout (plus 15 mins to avoid deleting local tempfiles
             # before the remote replicator kills it's rsync)
-            default = replicator_rsync_timeout + 900
+            default_rsync_timeout = replicator_rsync_timeout + 900
+            # there's not really a good reason to assume the replicator
+            # section's reclaim_age is more appropriate than the reconstructor
+            # reclaim_age - but we're already parsing the config so we can set
+            # the default value in our config if it's not already set
+            if 'reclaim_age' in replicator_config:
+                conf.setdefault('reclaim_age',
+                                replicator_config['reclaim_age'])
         self.rsync_tempfile_timeout = config_auto_int_value(
-            self.conf.get('rsync_tempfile_timeout'), default)
+            self.conf.get('rsync_tempfile_timeout'), default_rsync_timeout)
+        self.diskfile_router = diskfile.DiskFileRouter(conf, self.logger)
 
         self.auditor_type = 'ALL'
         self.zero_byte_only_at_fps = zero_byte_only_at_fps
@@ -104,8 +113,10 @@ class AuditorWorker(object):
                 description = _(' - parallel, %s') % device_dir_str
             else:
                 description = _(' - %s') % device_dir_str
-        self.logger.info(_('Begin object audit "%s" mode (%s%s)') %
-                         (mode, self.auditor_type, description))
+        self.logger.info(_('Begin object audit "%(mode)s" mode (%(audi_type)s'
+                           '%(description)s)') %
+                         {'mode': mode, 'audi_type': self.auditor_type,
+                          'description': description})
         begin = reported = time.time()
         self.total_bytes_processed = 0
         self.total_files_processed = 0
@@ -249,19 +260,27 @@ class AuditorWorker(object):
                             incr_by=chunk_len)
                         self.bytes_processed += chunk_len
                         self.total_bytes_processed += chunk_len
-        except DiskFileNotExist:
-            pass
         except DiskFileQuarantined as err:
             self.quarantines += 1
             self.logger.error(_('ERROR Object %(obj)s failed audit and was'
                                 ' quarantined: %(err)s'),
                               {'obj': location, 'err': err})
+        except DiskFileDeleted:
+            # If there is a reclaimable tombstone, we'll invalidate the hash
+            # to trigger the replicator to rehash/cleanup this suffix
+            ts = df._ondisk_info['ts_info']['timestamp']
+            if (not self.zero_byte_only_at_fps and
+                    (time.time() - float(ts)) > df.manager.reclaim_age):
+                df.manager.invalidate_hash(dirname(df._datadir))
+        except DiskFileNotExist:
+            pass
+
         self.passes += 1
         # _ondisk_info attr is initialized to None and filled in by open
         ondisk_info_dict = df._ondisk_info or {}
         if 'unexpected' in ondisk_info_dict:
             is_rsync_tempfile = lambda fpath: RE_RSYNC_TEMPFILE.match(
-                os.path.basename(fpath))
+                basename(fpath))
             rsync_tempfile_paths = filter(is_rsync_tempfile,
                                           ondisk_info_dict['unexpected'])
             mtime = time.time() - self.rsync_tempfile_timeout
@@ -280,7 +299,7 @@ class ObjectAuditor(Daemon):
             conf.get('zero_byte_files_per_second', 50))
         self.recon_cache_path = conf.get('recon_cache_path',
                                          '/var/cache/swift')
-        self.rcache = os.path.join(self.recon_cache_path, "object.recon")
+        self.rcache = join(self.recon_cache_path, "object.recon")
         self.interval = int(conf.get('interval', 30))
 
     def _sleep(self):
@@ -328,13 +347,13 @@ class ObjectAuditor(Daemon):
             kwargs['zero_byte_fps'] = zbo_fps
             self.run_audit(**kwargs)
         else:
-            pids = []
+            pids = set()
             if self.conf_zero_byte_fps:
                 zbf_pid = self.fork_child(zero_byte_fps=True, **kwargs)
-                pids.append(zbf_pid)
+                pids.add(zbf_pid)
             if self.concurrency == 1:
                 # Audit all devices in 1 process
-                pids.append(self.fork_child(**kwargs))
+                pids.add(self.fork_child(**kwargs))
             else:
                 # Divide devices amongst parallel processes set by
                 # self.concurrency.  Total number of parallel processes
@@ -348,7 +367,7 @@ class ObjectAuditor(Daemon):
                     pid = None
                     if len(pids) == parallel_proc:
                         pid = os.wait()[0]
-                        pids.remove(pid)
+                        pids.discard(pid)
 
                     if self.conf_zero_byte_fps and pid == zbf_pid and once:
                         # If we're only running one pass and the ZBF scanner
@@ -361,10 +380,10 @@ class ObjectAuditor(Daemon):
                         # sleep between ZBF scanner forks
                         self._sleep()
                         zbf_pid = self.fork_child(zero_byte_fps=True, **kwargs)
-                        pids.append(zbf_pid)
+                        pids.add(zbf_pid)
                     else:
                         kwargs['device_dirs'] = [device_list.pop()]
-                        pids.append(self.fork_child(**kwargs))
+                        pids.add(self.fork_child(**kwargs))
             while pids:
                 pid = os.wait()[0]
                 # ZBF scanner must be restarted as soon as it finishes
@@ -375,8 +394,8 @@ class ObjectAuditor(Daemon):
                     # sleep between ZBF scanner forks
                     self._sleep()
                     zbf_pid = self.fork_child(zero_byte_fps=True, **kwargs)
-                    pids.append(zbf_pid)
-                pids.remove(pid)
+                    pids.add(zbf_pid)
+                pids.discard(pid)
 
     def run_forever(self, *args, **kwargs):
         """Run the object audit until stopped."""
