@@ -68,7 +68,7 @@ def _get_partners(frag_index, part_nodes):
 
 class RebuildingECDiskFileStream(object):
     """
-    This class wraps the the reconstructed fragment archive data and
+    This class wraps the reconstructed fragment archive data and
     metadata in the DiskFile interface for ssync.
     """
 
@@ -77,7 +77,8 @@ class RebuildingECDiskFileStream(object):
         self.datafile_metadata = datafile_metadata
 
         # the new FA is going to have the same length as others in the set
-        self._content_length = int(self.datafile_metadata['Content-Length'])
+        self._content_length = self.datafile_metadata['Content-Length']
+
         # update the FI and delete the ETag, the obj server will
         # recalc on the other side...
         self.datafile_metadata['X-Object-Sysmeta-Ec-Frag-Index'] = frag_index
@@ -126,7 +127,7 @@ class ObjectReconstructor(Daemon):
         self.bind_ip = conf.get('bind_ip', '0.0.0.0')
         self.servers_per_port = int(conf.get('servers_per_port', '0') or 0)
         self.port = None if self.servers_per_port else \
-            int(conf.get('bind_port', 6000))
+            int(conf.get('bind_port', 6200))
         self.concurrency = int(conf.get('concurrency', 1))
         self.stats_interval = int(conf.get('stats_interval', '300'))
         self.ring_check_interval = int(conf.get('ring_check_interval', 15))
