@@ -46,10 +46,6 @@ DEFAULT_RSYNC_TIMEOUT = 900
 hubs.use_hub(get_hub())
 
 
-def _do_listdir(partition, replication_cycle):
-    return (((partition + replication_cycle) % 10) == 0)
-
-
 class ObjectReplicator(Daemon):
     """
     Replicate objects.
@@ -72,13 +68,12 @@ class ObjectReplicator(Daemon):
         self.bind_ip = conf.get('bind_ip', '0.0.0.0')
         self.servers_per_port = int(conf.get('servers_per_port', '0') or 0)
         self.port = None if self.servers_per_port else \
-            int(conf.get('bind_port', 6000))
+            int(conf.get('bind_port', 6200))
         self.concurrency = int(conf.get('concurrency', 1))
         self.stats_interval = int(conf.get('stats_interval', '300'))
         self.ring_check_interval = int(conf.get('ring_check_interval', 15))
         self.next_check = time.time() + self.ring_check_interval
         self.reclaim_age = int(conf.get('reclaim_age', 86400 * 7))
-        self.replication_cycle = random.randint(0, 9)
         self.partition_times = []
         self.interval = int(conf.get('interval') or
                             conf.get('run_pause') or 30)
@@ -206,12 +201,9 @@ class ObjectReplicator(Daemon):
             if self.rsync_error_log_line_length:
                 error_line = error_line[:self.rsync_error_log_line_length]
             self.logger.error(error_line)
-        elif results:
-            self.logger.info(
-                _("Successful rsync of %(src)s at %(dst)s (%(time).03f)"),
-                {'src': args[-2], 'dst': args[-1], 'time': total_time})
         else:
-            self.logger.debug(
+            log_method = self.logger.info if results else self.logger.debug
+            log_method(
                 _("Successful rsync of %(src)s at %(dst)s (%(time).03f)"),
                 {'src': args[-2], 'dst': args[-1], 'time': total_time})
         return ret_val
@@ -234,6 +226,7 @@ class ObjectReplicator(Daemon):
             '--timeout=%s' % self.rsync_io_timeout,
             '--contimeout=%s' % self.rsync_io_timeout,
             '--bwlimit=%s' % self.rsync_bwlimit,
+            '--exclude=.*.%s' % ''.join('[0-9a-zA-Z]' for i in range(6))
         ]
         if self.rsync_compress and \
                 job['region'] != node['region']:
@@ -411,9 +404,7 @@ class ObjectReplicator(Daemon):
         try:
             hashed, local_hash = tpool_reraise(
                 self._diskfile_mgr._get_hashes, job['path'],
-                do_listdir=_do_listdir(
-                    int(job['partition']),
-                    self.replication_cycle),
+                do_listdir=(self.replication_count % 10) == 0,
                 reclaim_age=self.reclaim_age)
             self.suffix_hash += hashed
             self.logger.update_stats('suffix.hashes', hashed)
@@ -440,8 +431,9 @@ class ObjectReplicator(Daemon):
                             node['device'], job['partition'], 'REPLICATE',
                             '', headers=headers).getresponse()
                         if resp.status == HTTP_INSUFFICIENT_STORAGE:
-                            self.logger.error(_('%(ip)s/%(device)s responded'
-                                                ' as unmounted'), node)
+                            self.logger.error(
+                                _('%(replication_ip)s/%(device)s '
+                                  'responded as unmounted'), node)
                             attempts_left += 1
                             failure_devs_info.add((node['replication_ip'],
                                                    node['device']))
@@ -653,9 +645,10 @@ class ObjectReplicator(Daemon):
                              if failure_dev])
                     continue
         if not found_local:
-            self.logger.error("Can't find itself %s with port %s in ring "
-                              "file, not replicating",
-                              ", ".join(ips), self.port)
+            self.logger.error("Can't find itself in policy with index %d with"
+                              " ips %s and with port %s in ring file, not"
+                              " replicating",
+                              int(policy), ", ".join(ips), self.port)
         return jobs
 
     def collect_jobs(self, override_devices=None, override_partitions=None,
@@ -699,7 +692,6 @@ class ObjectReplicator(Daemon):
         self.suffix_hash = 0
         self.replication_count = 0
         self.last_replication_count = -1
-        self.replication_cycle = (self.replication_cycle + 1) % 10
         self.partition_times = []
         self.my_replication_ips = self._get_my_replication_ips()
         self.all_devs_info = set()
