@@ -58,18 +58,53 @@ def get_server_number(ipport, ipport2server):
     return server, number
 
 
-def start_server(ipport, ipport2server, pids, check=True):
+def start_server(ipport, ipport2server):
     server, number = get_server_number(ipport, ipport2server)
     err = Manager([server]).start(number=number, wait=False)
     if err:
         raise Exception('unable to start %s' % (
             server if not number else '%s%s' % (server, number)))
-    if check:
-        return check_server(ipport, ipport2server, pids)
-    return None
+    return check_server(ipport, ipport2server)
 
 
-def check_server(ipport, ipport2server, pids, timeout=CHECK_SERVER_TIMEOUT):
+def _check_storage(ipport, path):
+    conn = HTTPConnection(*ipport)
+    conn.request('GET', path)
+    resp = conn.getresponse()
+    # 404 because it's a nonsense path (and mount_check is false)
+    # 507 in case the test target is a VM using mount_check
+    if resp.status not in (404, 507):
+        raise Exception(
+            'Unexpected status %s' % resp.status)
+    return resp
+
+
+def _check_proxy(ipport, user, key):
+    url, token = get_auth('http://%s:%d/auth/v1.0' % ipport,
+                          user, key)
+    account = url.split('/')[-1]
+    head_account(url, token)
+    return url, token, account
+
+
+def _retry_timeout(f, args=None, kwargs=None, timeout=CHECK_SERVER_TIMEOUT):
+    args = args or ()
+    kwargs = kwargs or {}
+    try_until = time() + timeout
+    while True:
+        try:
+            return f(*args, **kwargs)
+        except Exception as err:
+            if time() > try_until:
+                print(err)
+                fsignature = '%s(*%r, **%r)' % (f.__name__, args, kwargs)
+                print('Giving up on %s after %s seconds.' % (
+                    fsignature, timeout))
+                raise err
+            sleep(0.1)
+
+
+def check_server(ipport, ipport2server):
     server = ipport2server[ipport]
     if server[:-1] in ('account', 'container', 'object'):
         if int(server[-1]) > 4:
@@ -79,44 +114,14 @@ def check_server(ipport, ipport2server, pids, timeout=CHECK_SERVER_TIMEOUT):
             path += '/3'
         elif server[:-1] == 'object':
             path += '/3/4'
-        try_until = time() + timeout
-        while True:
-            try:
-                conn = HTTPConnection(*ipport)
-                conn.request('GET', path)
-                resp = conn.getresponse()
-                # 404 because it's a nonsense path (and mount_check is false)
-                # 507 in case the test target is a VM using mount_check
-                if resp.status not in (404, 507):
-                    raise Exception(
-                        'Unexpected status %s' % resp.status)
-                break
-            except Exception as err:
-                if time() > try_until:
-                    print(err)
-                    print('Giving up on %s:%s after %s seconds.' % (
-                        server, ipport, timeout))
-                    raise err
-                sleep(0.1)
+        rv = _retry_timeout(_check_storage, args=(ipport, path))
     else:
-        try_until = time() + timeout
-        while True:
-            try:
-                url, token = get_auth('http://%s:%d/auth/v1.0' % ipport,
-                                      'test:tester', 'testing')
-                account = url.split('/')[-1]
-                head_account(url, token)
-                return url, token, account
-            except Exception as err:
-                if time() > try_until:
-                    print(err)
-                    print('Giving up on proxy:8080 after 30 seconds.')
-                    raise err
-                sleep(0.1)
-    return None
+        rv = _retry_timeout(_check_proxy, args=(
+            ipport, 'test:tester', 'testing'))
+    return rv
 
 
-def kill_server(ipport, ipport2server, pids):
+def kill_server(ipport, ipport2server):
     server, number = get_server_number(ipport, ipport2server)
     err = Manager([server]).kill(number=number)
     if err:
@@ -136,7 +141,7 @@ def kill_server(ipport, ipport2server, pids):
         sleep(0.1)
 
 
-def kill_nonprimary_server(primary_nodes, ipport2server, pids):
+def kill_nonprimary_server(primary_nodes, ipport2server):
     primary_ipports = [(n['ip'], n['port']) for n in primary_nodes]
     for ipport, server in ipport2server.items():
         if ipport in primary_ipports:
@@ -146,7 +151,7 @@ def kill_nonprimary_server(primary_nodes, ipport2server, pids):
         raise Exception('Cannot figure out server type for %r' % primary_nodes)
     for ipport, server in list(ipport2server.items()):
         if server[:-1] == server_type and ipport not in primary_ipports:
-            kill_server(ipport, ipport2server, pids)
+            kill_server(ipport, ipport2server)
             return ipport
 
 
@@ -206,7 +211,7 @@ def get_ring(ring_name, required_replicas, required_devices,
     devs = [dev for dev in ring.devs if dev is not None]
     if len(devs) != required_devices:
         raise SkipTest('%s has %s devices instead of %s' % (
-            ring.serialized_path, len(ring.devs), required_devices))
+            ring.serialized_path, len(devs), required_devices))
     for dev in devs:
         # verify server is exposing mounted device
         ipport = (dev['ip'], dev['port'])
@@ -268,6 +273,11 @@ def get_policy(**kwargs):
 def resetswift():
     p = Popen("resetswift 2>&1", shell=True, stdout=PIPE)
     stdout, _stderr = p.communicate()
+    if p.returncode:
+        raise AssertionError(
+            'Cleanup with "resetswift" failed: stdout: %s, stderr: %s'
+            % (stdout, _stderr))
+
     print(stdout)
     Manager(['all']).stop()
 
@@ -322,7 +332,6 @@ class ProbeTest(unittest.TestCase):
 
     def setUp(self):
         resetswift()
-        self.pids = {}
         try:
             self.ipport2server = {}
             self.configs = defaultdict(dict)
@@ -354,11 +363,19 @@ class ProbeTest(unittest.TestCase):
 
             Manager(['main']).start(wait=False)
             for ipport in self.ipport2server:
-                check_server(ipport, self.ipport2server, self.pids)
+                check_server(ipport, self.ipport2server)
             proxy_ipport = ('127.0.0.1', 8080)
             self.ipport2server[proxy_ipport] = 'proxy'
             self.url, self.token, self.account = check_server(
-                proxy_ipport, self.ipport2server, self.pids)
+                proxy_ipport, self.ipport2server)
+            self.account_1 = {
+                'url': self.url, 'token': self.token, 'account': self.account}
+
+            rv = _retry_timeout(_check_proxy, args=(
+                proxy_ipport, 'test2:tester2', 'testing2'))
+            self.account_2 = {
+                k: v for (k, v) in zip(('url', 'token', 'account'), rv)}
+
             self.replicators = Manager(
                 ['account-replicator', 'container-replicator',
                  'object-replicator'])
@@ -443,10 +460,13 @@ class ProbeTest(unittest.TestCase):
             swift_dir = /etc/swift
 
             [pipeline:main]
-            pipeline = catch_errors cache proxy-server
+            pipeline = catch_errors cache copy proxy-server
 
             [app:proxy-server]
             use = egg:swift#proxy
+
+            [filter:copy]
+            use = egg:swift#copy
             object_post_as_copy = %s
 
             [filter:cache]
