@@ -15,8 +15,9 @@
 
 import unittest
 
-from swift.common.swob import Request, HTTPUnauthorized
-from swift.common.middleware import container_quotas
+from swift.common.swob import Request, HTTPUnauthorized, HTTPOk, HTTPException
+from swift.common.middleware import container_quotas, copy
+from test.unit.common.middleware.helpers import FakeSwift
 
 
 class FakeCache(object):
@@ -95,32 +96,6 @@ class TestContainerQuotas(unittest.TestCase):
         self.assertEqual(res.status_int, 413)
         self.assertEqual(res.body, 'Upload exceeds quota.')
 
-    def test_exceed_bytes_quota_copy_from(self):
-        app = container_quotas.ContainerQuotaMiddleware(FakeApp(), {})
-        cache = FakeCache({'bytes': 0, 'meta': {'quota-bytes': '2'}})
-
-        req = Request.blank('/v1/a/c/o',
-                            environ={'REQUEST_METHOD': 'PUT',
-                                     'swift.object/a/c2/o2': {'length': 10},
-                                     'swift.cache': cache},
-                            headers={'x-copy-from': '/c2/o2'})
-        res = req.get_response(app)
-        self.assertEqual(res.status_int, 413)
-        self.assertEqual(res.body, 'Upload exceeds quota.')
-
-    def test_exceed_bytes_quota_copy_verb(self):
-        app = container_quotas.ContainerQuotaMiddleware(FakeApp(), {})
-        cache = FakeCache({'bytes': 0, 'meta': {'quota-bytes': '2'}})
-
-        req = Request.blank('/v1/a/c2/o2',
-                            environ={'REQUEST_METHOD': 'COPY',
-                                     'swift.object/a/c2/o2': {'length': 10},
-                                     'swift.cache': cache},
-                            headers={'Destination': '/c/o'})
-        res = req.get_response(app)
-        self.assertEqual(res.status_int, 413)
-        self.assertEqual(res.body, 'Upload exceeds quota.')
-
     def test_not_exceed_bytes_quota(self):
         app = container_quotas.ContainerQuotaMiddleware(FakeApp(), {})
         cache = FakeCache({'bytes': 0, 'meta': {'quota-bytes': '100'}})
@@ -131,60 +106,6 @@ class TestContainerQuotas(unittest.TestCase):
         res = req.get_response(app)
         self.assertEqual(res.status_int, 200)
 
-    def test_not_exceed_bytes_quota_copy_from(self):
-        app = container_quotas.ContainerQuotaMiddleware(FakeApp(), {})
-        cache = FakeCache({'bytes': 0, 'meta': {'quota-bytes': '100'}})
-        req = Request.blank('/v1/a/c/o',
-                            environ={'REQUEST_METHOD': 'PUT',
-                                     'swift.object/a/c2/o2': {'length': 10},
-                                     'swift.cache': cache},
-                            headers={'x-copy-from': '/c2/o2'})
-        res = req.get_response(app)
-        self.assertEqual(res.status_int, 200)
-
-    def test_not_exceed_bytes_quota_copy_verb(self):
-        app = container_quotas.ContainerQuotaMiddleware(FakeApp(), {})
-        cache = FakeCache({'bytes': 0, 'meta': {'quota-bytes': '100'}})
-        req = Request.blank('/v1/a/c2/o2',
-                            environ={'REQUEST_METHOD': 'COPY',
-                                     'swift.object/a/c2/o2': {'length': 10},
-                                     'swift.cache': cache},
-                            headers={'Destination': '/c/o'})
-        res = req.get_response(app)
-        self.assertEqual(res.status_int, 200)
-
-    def test_bytes_quota_copy_from_no_src(self):
-        app = container_quotas.ContainerQuotaMiddleware(FakeApp(), {})
-        cache = FakeCache({'bytes': 0, 'meta': {'quota-bytes': '100'}})
-        req = Request.blank('/v1/a/c/o',
-                            environ={'REQUEST_METHOD': 'PUT',
-                                     'swift.object/a/c2/o2': {'length': 10},
-                                     'swift.cache': cache},
-                            headers={'x-copy-from': '/c2/o3'})
-        res = req.get_response(app)
-        self.assertEqual(res.status_int, 200)
-
-    def test_bytes_quota_copy_from_bad_src(self):
-        app = container_quotas.ContainerQuotaMiddleware(FakeApp(), {})
-        cache = FakeCache({'bytes': 0, 'meta': {'quota-bytes': '100'}})
-        req = Request.blank('/v1/a/c/o',
-                            environ={'REQUEST_METHOD': 'PUT',
-                                     'swift.cache': cache},
-                            headers={'x-copy-from': 'bad_path'})
-        res = req.get_response(app)
-        self.assertEqual(res.status_int, 412)
-
-    def test_bytes_quota_copy_verb_no_src(self):
-        app = container_quotas.ContainerQuotaMiddleware(FakeApp(), {})
-        cache = FakeCache({'bytes': 0, 'meta': {'quota-bytes': '100'}})
-        req = Request.blank('/v1/a/c2/o3',
-                            environ={'REQUEST_METHOD': 'COPY',
-                                     'swift.object/a/c2/o2': {'length': 10},
-                                     'swift.cache': cache},
-                            headers={'Destination': '/c/o'})
-        res = req.get_response(app)
-        self.assertEqual(res.status_int, 200)
-
     def test_exceed_counts_quota(self):
         app = container_quotas.ContainerQuotaMiddleware(FakeApp(), {})
         cache = FakeCache({'object_count': 1, 'meta': {'quota-count': '1'}})
@@ -196,61 +117,6 @@ class TestContainerQuotas(unittest.TestCase):
         self.assertEqual(res.status_int, 413)
         self.assertEqual(res.body, 'Upload exceeds quota.')
 
-    def test_exceed_counts_quota_copy_from(self):
-        app = container_quotas.ContainerQuotaMiddleware(FakeApp(), {})
-        cache = FakeCache({'object_count': 1, 'meta': {'quota-count': '1'}})
-        req = Request.blank('/v1/a/c/o',
-                            environ={'REQUEST_METHOD': 'PUT',
-                                     'swift.object/a/c2/o2': {'length': 10},
-                                     'swift.cache': cache},
-                            headers={'x-copy-from': '/c2/o2'})
-        res = req.get_response(app)
-        self.assertEqual(res.status_int, 413)
-        self.assertEqual(res.body, 'Upload exceeds quota.')
-
-    def test_exceed_counts_quota_copy_verb(self):
-        app = container_quotas.ContainerQuotaMiddleware(FakeApp(), {})
-        cache = FakeCache({'object_count': 1, 'meta': {'quota-count': '1'}})
-        req = Request.blank('/v1/a/c2/o2',
-                            environ={'REQUEST_METHOD': 'COPY',
-                                     'swift.cache': cache},
-                            headers={'Destination': '/c/o'})
-        res = req.get_response(app)
-        self.assertEqual(res.status_int, 413)
-        self.assertEqual(res.body, 'Upload exceeds quota.')
-
-    def test_exceed_counts_quota_copy_cross_account_verb(self):
-        app = container_quotas.ContainerQuotaMiddleware(FakeApp(), {})
-        a_c_cache = {'storage_policy': '0', 'meta': {'quota-count': '2'},
-                     'status': 200, 'object_count': 1}
-        a2_c_cache = {'storage_policy': '0', 'meta': {'quota-count': '1'},
-                      'status': 200, 'object_count': 1}
-        req = Request.blank('/v1/a/c2/o2',
-                            environ={'REQUEST_METHOD': 'COPY',
-                                     'swift.container/a/c': a_c_cache,
-                                     'swift.container/a2/c': a2_c_cache},
-                            headers={'Destination': '/c/o',
-                                     'Destination-Account': 'a2'})
-        res = req.get_response(app)
-        self.assertEqual(res.status_int, 413)
-        self.assertEqual(res.body, 'Upload exceeds quota.')
-
-    def test_exceed_counts_quota_copy_cross_account_PUT_verb(self):
-        app = container_quotas.ContainerQuotaMiddleware(FakeApp(), {})
-        a_c_cache = {'storage_policy': '0', 'meta': {'quota-count': '2'},
-                     'status': 200, 'object_count': 1}
-        a2_c_cache = {'storage_policy': '0', 'meta': {'quota-count': '1'},
-                      'status': 200, 'object_count': 1}
-        req = Request.blank('/v1/a2/c/o',
-                            environ={'REQUEST_METHOD': 'PUT',
-                                     'swift.container/a/c': a_c_cache,
-                                     'swift.container/a2/c': a2_c_cache},
-                            headers={'X-Copy-From': '/c2/o2',
-                                     'X-Copy-From-Account': 'a'})
-        res = req.get_response(app)
-        self.assertEqual(res.status_int, 413)
-        self.assertEqual(res.body, 'Upload exceeds quota.')
-
     def test_not_exceed_counts_quota(self):
         app = container_quotas.ContainerQuotaMiddleware(FakeApp(), {})
         cache = FakeCache({'object_count': 1, 'meta': {'quota-count': '2'}})
@@ -261,26 +127,6 @@ class TestContainerQuotas(unittest.TestCase):
         res = req.get_response(app)
         self.assertEqual(res.status_int, 200)
 
-    def test_not_exceed_counts_quota_copy_from(self):
-        app = container_quotas.ContainerQuotaMiddleware(FakeApp(), {})
-        cache = FakeCache({'object_count': 1, 'meta': {'quota-count': '2'}})
-        req = Request.blank('/v1/a/c/o',
-                            environ={'REQUEST_METHOD': 'PUT',
-                                     'swift.cache': cache},
-                            headers={'x-copy-from': '/c2/o2'})
-        res = req.get_response(app)
-        self.assertEqual(res.status_int, 200)
-
-    def test_not_exceed_counts_quota_copy_verb(self):
-        app = container_quotas.ContainerQuotaMiddleware(FakeApp(), {})
-        cache = FakeCache({'object_count': 1, 'meta': {'quota-count': '2'}})
-        req = Request.blank('/v1/a/c2/o2',
-                            environ={'REQUEST_METHOD': 'COPY',
-                                     'swift.cache': cache},
-                            headers={'Destination': '/c/o'})
-        res = req.get_response(app)
-        self.assertEqual(res.status_int, 200)
-
     def test_invalid_quotas(self):
         req = Request.blank(
             '/v1/a/c',
@@ -346,5 +192,170 @@ class TestContainerQuotas(unittest.TestCase):
         res = req.get_response(app)
         self.assertEqual(res.status_int, 401)
 
+
+class ContainerQuotaCopyingTestCases(unittest.TestCase):
+
+    def setUp(self):
+        self.app = FakeSwift()
+        self.cq_filter = container_quotas.filter_factory({})(self.app)
+        self.copy_filter = copy.filter_factory({})(self.cq_filter)
+
+    def test_exceed_bytes_quota_copy_verb(self):
+        cache = FakeCache({'bytes': 0, 'meta': {'quota-bytes': '2'}})
+        self.app.register('GET', '/v1/a/c2/o2', HTTPOk,
+                          {'Content-Length': '10'}, 'passed')
+
+        req = Request.blank('/v1/a/c2/o2',
+                            environ={'REQUEST_METHOD': 'COPY',
+                                     'swift.cache': cache},
+                            headers={'Destination': '/c/o'})
+        res = req.get_response(self.copy_filter)
+        self.assertEqual(res.status_int, 413)
+        self.assertEqual(res.body, 'Upload exceeds quota.')
+
+    def test_not_exceed_bytes_quota_copy_verb(self):
+        self.app.register('GET', '/v1/a/c2/o2', HTTPOk,
+                          {'Content-Length': '10'}, 'passed')
+        self.app.register(
+            'PUT', '/v1/a/c/o', HTTPOk, {}, 'passed')
+        cache = FakeCache({'bytes': 0, 'meta': {'quota-bytes': '100'}})
+        req = Request.blank('/v1/a/c2/o2',
+                            environ={'REQUEST_METHOD': 'COPY',
+                                     'swift.cache': cache},
+                            headers={'Destination': '/c/o'})
+        res = req.get_response(self.copy_filter)
+        self.assertEqual(res.status_int, 200)
+
+    def test_exceed_counts_quota_copy_verb(self):
+        self.app.register('GET', '/v1/a/c2/o2', HTTPOk, {}, 'passed')
+        cache = FakeCache({'object_count': 1, 'meta': {'quota-count': '1'}})
+        req = Request.blank('/v1/a/c2/o2',
+                            environ={'REQUEST_METHOD': 'COPY',
+                                     'swift.cache': cache},
+                            headers={'Destination': '/c/o'})
+        res = req.get_response(self.copy_filter)
+        self.assertEqual(res.status_int, 413)
+        self.assertEqual(res.body, 'Upload exceeds quota.')
+
+    def test_exceed_counts_quota_copy_cross_account_verb(self):
+        self.app.register('GET', '/v1/a/c2/o2', HTTPOk, {}, 'passed')
+        a_c_cache = {'storage_policy': '0', 'meta': {'quota-count': '2'},
+                     'status': 200, 'object_count': 1}
+        a2_c_cache = {'storage_policy': '0', 'meta': {'quota-count': '1'},
+                      'status': 200, 'object_count': 1}
+        req = Request.blank('/v1/a/c2/o2',
+                            environ={'REQUEST_METHOD': 'COPY',
+                                     'swift.infocache': {
+                                         'container/a/c': a_c_cache,
+                                         'container/a2/c': a2_c_cache}},
+                            headers={'Destination': '/c/o',
+                                     'Destination-Account': 'a2'})
+        res = req.get_response(self.copy_filter)
+        self.assertEqual(res.status_int, 413)
+        self.assertEqual(res.body, 'Upload exceeds quota.')
+
+    def test_exceed_counts_quota_copy_cross_account_PUT_verb(self):
+        self.app.register('GET', '/v1/a/c2/o2', HTTPOk, {}, 'passed')
+        a_c_cache = {'storage_policy': '0', 'meta': {'quota-count': '2'},
+                     'status': 200, 'object_count': 1}
+        a2_c_cache = {'storage_policy': '0', 'meta': {'quota-count': '1'},
+                      'status': 200, 'object_count': 1}
+        req = Request.blank('/v1/a2/c/o',
+                            environ={'REQUEST_METHOD': 'PUT',
+                                     'swift.infocache': {
+                                         'container/a/c': a_c_cache,
+                                         'container/a2/c': a2_c_cache}},
+                            headers={'X-Copy-From': '/c2/o2',
+                                     'X-Copy-From-Account': 'a'})
+        res = req.get_response(self.copy_filter)
+        self.assertEqual(res.status_int, 413)
+        self.assertEqual(res.body, 'Upload exceeds quota.')
+
+    def test_exceed_bytes_quota_copy_from(self):
+        self.app.register('GET', '/v1/a/c2/o2', HTTPOk,
+                          {'Content-Length': '10'}, 'passed')
+        cache = FakeCache({'bytes': 0, 'meta': {'quota-bytes': '2'}})
+
+        req = Request.blank('/v1/a/c/o',
+                            environ={'REQUEST_METHOD': 'PUT',
+                                     'swift.cache': cache},
+                            headers={'x-copy-from': '/c2/o2'})
+        res = req.get_response(self.copy_filter)
+        self.assertEqual(res.status_int, 413)
+        self.assertEqual(res.body, 'Upload exceeds quota.')
+
+    def test_not_exceed_bytes_quota_copy_from(self):
+        self.app.register('GET', '/v1/a/c2/o2', HTTPOk,
+                          {'Content-Length': '10'}, 'passed')
+        self.app.register(
+            'PUT', '/v1/a/c/o', HTTPOk, {}, 'passed')
+        cache = FakeCache({'bytes': 0, 'meta': {'quota-bytes': '100'}})
+        req = Request.blank('/v1/a/c/o',
+                            environ={'REQUEST_METHOD': 'PUT',
+                                     'swift.cache': cache},
+                            headers={'x-copy-from': '/c2/o2'})
+        res = req.get_response(self.copy_filter)
+        self.assertEqual(res.status_int, 200)
+
+    def test_bytes_quota_copy_from_no_src(self):
+        self.app.register('GET', '/v1/a/c2/o3', HTTPOk, {}, 'passed')
+        self.app.register(
+            'PUT', '/v1/a/c/o', HTTPOk, {}, 'passed')
+        cache = FakeCache({'bytes': 0, 'meta': {'quota-bytes': '100'}})
+        req = Request.blank('/v1/a/c/o',
+                            environ={'REQUEST_METHOD': 'PUT',
+                                     'swift.cache': cache},
+                            headers={'x-copy-from': '/c2/o3'})
+        res = req.get_response(self.copy_filter)
+        self.assertEqual(res.status_int, 200)
+
+    def test_bytes_quota_copy_from_bad_src(self):
+        cache = FakeCache({'bytes': 0, 'meta': {'quota-bytes': '100'}})
+        req = Request.blank('/v1/a/c/o',
+                            environ={'REQUEST_METHOD': 'PUT',
+                                     'swift.cache': cache},
+                            headers={'x-copy-from': 'bad_path'})
+        with self.assertRaises(HTTPException) as catcher:
+            req.get_response(self.copy_filter)
+        self.assertEqual(412, catcher.exception.status_int)
+
+    def test_exceed_counts_quota_copy_from(self):
+        self.app.register('GET', '/v1/a/c2/o2', HTTPOk,
+                          {'Content-Length': '10'}, 'passed')
+        cache = FakeCache({'object_count': 1, 'meta': {'quota-count': '1'}})
+        req = Request.blank('/v1/a/c/o',
+                            environ={'REQUEST_METHOD': 'PUT',
+                                     'swift.cache': cache},
+                            headers={'x-copy-from': '/c2/o2'})
+        res = req.get_response(self.copy_filter)
+        self.assertEqual(res.status_int, 413)
+        self.assertEqual(res.body, 'Upload exceeds quota.')
+
+    def test_not_exceed_counts_quota_copy_from(self):
+        self.app.register('GET', '/v1/a/c2/o2', HTTPOk,
+                          {'Content-Length': '10'}, 'passed')
+        self.app.register(
+            'PUT', '/v1/a/c/o', HTTPOk, {}, 'passed')
+        cache = FakeCache({'object_count': 1, 'meta': {'quota-count': '2'}})
+        req = Request.blank('/v1/a/c/o',
+                            environ={'REQUEST_METHOD': 'PUT',
+                                     'swift.cache': cache},
+                            headers={'x-copy-from': '/c2/o2'})
+        res = req.get_response(self.copy_filter)
+        self.assertEqual(res.status_int, 200)
+
+    def test_not_exceed_counts_quota_copy_verb(self):
+        self.app.register('GET', '/v1/a/c2/o2', HTTPOk,
+                          {'Content-Length': '10'}, 'passed')
+        self.app.register(
+            'PUT', '/v1/a/c/o', HTTPOk, {}, 'passed')
+        cache = FakeCache({'object_count': 1, 'meta': {'quota-count': '2'}})
+        req = Request.blank('/v1/a/c2/o2',
+                            environ={'REQUEST_METHOD': 'COPY',
+                                     'swift.cache': cache},
+                            headers={'Destination': '/c/o'})
+        res = req.get_response(self.copy_filter)
+        self.assertEqual(res.status_int, 200)
+
 if __name__ == '__main__':
     unittest.main()
