@@ -30,9 +30,10 @@
 
 import hmac
 import itertools
+import mock
 import unittest
 from hashlib import sha1
-from time import time
+from time import time, strftime, gmtime
 
 from swift.common.middleware import tempauth, tempurl
 from swift.common.header_key_dict import HeaderKeyDict
@@ -96,7 +97,8 @@ class TestTempURL(unittest.TestCase):
             if key:
                 meta[meta_name] = key
 
-        environ['swift.account/' + account] = {
+        ic = environ.setdefault('swift.infocache', {})
+        ic['account/' + account] = {
             'status': 204,
             'container_count': '0',
             'total_object_count': '0',
@@ -108,8 +110,8 @@ class TestTempURL(unittest.TestCase):
             meta_name = 'Temp-URL-key' + (("-%d" % (i + 1) if i else ""))
             meta[meta_name] = key
 
-        container_cache_key = 'swift.container/' + account + '/c'
-        environ.setdefault(container_cache_key, {'meta': meta})
+        container_cache_key = 'container/' + account + '/c'
+        ic.setdefault(container_cache_key, {'meta': meta})
 
     def test_passthrough(self):
         resp = self._make_request('/v1/a/c/o').get_response(self.tempurl)
@@ -134,6 +136,9 @@ class TestTempURL(unittest.TestCase):
         self.assertEqual(resp.status_int, 200)
         self.assertEqual(resp.headers['content-disposition'],
                          'attachment; filename="o"; ' + "filename*=UTF-8''o")
+        self.assertEqual(resp.headers['expires'],
+                         strftime('%a, %d %b %Y %H:%M:%S GMT',
+                                  gmtime(expires)))
         self.assertEqual(req.environ['swift.authorize_override'], True)
         self.assertEqual(req.environ['REMOTE_USER'], '.wsgi.tempurl')
 
@@ -159,7 +164,8 @@ class TestTempURL(unittest.TestCase):
             self.assert_valid_sig(expires, path, [key1, key2], sig)
 
     def test_get_valid_container_keys(self):
-        environ = {}
+        ic = {}
+        environ = {'swift.infocache': ic}
         # Add two static container keys
         container_keys = ['me', 'other']
         meta = {}
@@ -167,7 +173,7 @@ class TestTempURL(unittest.TestCase):
             meta_name = 'Temp-URL-key' + (("-%d" % (idx + 1) if idx else ""))
             if key:
                 meta[meta_name] = key
-        environ['swift.container/a/c'] = {'meta': meta}
+        ic['container/a/c'] = {'meta': meta}
 
         method = 'GET'
         expires = int(time() + 86400)
@@ -181,9 +187,10 @@ class TestTempURL(unittest.TestCase):
         for sig in (sig1, sig2):
             self.assert_valid_sig(expires, path, account_keys, sig, environ)
 
-    def test_get_valid_with_filename(self):
+    @mock.patch('swift.common.middleware.tempurl.time', return_value=0)
+    def test_get_valid_with_filename(self, mock_time):
         method = 'GET'
-        expires = int(time() + 86400)
+        expires = (((24 + 1) * 60 + 1) * 60) + 1
         path = '/v1/a/c/o'
         key = 'abc'
         hmac_body = '%s\n%s\n%s' % (method, expires, path)
@@ -197,10 +204,31 @@ class TestTempURL(unittest.TestCase):
         self.assertEqual(resp.headers['content-disposition'],
                          'attachment; filename="bob %22killer%22.txt"; ' +
                          "filename*=UTF-8''bob%20%22killer%22.txt")
+        self.assertIn('expires', resp.headers)
+        self.assertEqual('Fri, 02 Jan 1970 01:01:01 GMT',
+                         resp.headers['expires'])
         self.assertEqual(req.environ['swift.authorize_override'], True)
         self.assertEqual(req.environ['REMOTE_USER'], '.wsgi.tempurl')
 
-    def test_head_valid(self):
+    def test_head_valid_with_filename(self):
+        method = 'HEAD'
+        expires = int(time() + 86400)
+        path = '/v1/a/c/o'
+        key = 'abc'
+        hmac_body = '%s\n%s\n%s' % (method, expires, path)
+        sig = hmac.new(key, hmac_body, sha1).hexdigest()
+        req = self._make_request(path, keys=[key], environ={
+            'REQUEST_METHOD': 'HEAD',
+            'QUERY_STRING': 'temp_url_sig=%s&temp_url_expires=%s&'
+            'filename=bob_killer.txt' % (sig, expires)})
+        self.tempurl.app = FakeApp(iter([('200 Ok', (), '123')]))
+        resp = req.get_response(self.tempurl)
+        self.assertEqual(resp.status_int, 200)
+        self.assertEqual(resp.headers['content-disposition'],
+                         'attachment; filename="bob_killer.txt"; ' +
+                         "filename*=UTF-8''bob_killer.txt")
+
+    def test_head_and_get_headers_match(self):
         method = 'HEAD'
         expires = int(time() + 86400)
         path = '/v1/a/c/o'
@@ -213,11 +241,22 @@ class TestTempURL(unittest.TestCase):
             % (sig, expires)})
         self.tempurl.app = FakeApp(iter([('200 Ok', (), '123')]))
         resp = req.get_response(self.tempurl)
-        self.assertEqual(resp.status_int, 200)
 
-    def test_get_valid_with_filename_and_inline(self):
+        get_method = 'GET'
+        get_hmac_body = '%s\n%s\n%s' % (get_method, expires, path)
+        get_sig = hmac.new(key, get_hmac_body, sha1).hexdigest()
+        get_req = self._make_request(path, keys=[key], environ={
+            'REQUEST_METHOD': 'GET',
+            'QUERY_STRING': 'temp_url_sig=%s&temp_url_expires=%s'
+            % (get_sig, expires)})
+        self.tempurl.app = FakeApp(iter([('200 Ok', (), '123')]))
+        get_resp = get_req.get_response(self.tempurl)
+        self.assertEqual(resp.headers, get_resp.headers)
+
+    @mock.patch('swift.common.middleware.tempurl.time', return_value=0)
+    def test_get_valid_with_filename_and_inline(self, mock_time):
         method = 'GET'
-        expires = int(time() + 86400)
+        expires = 1
         path = '/v1/a/c/o'
         key = 'abc'
         hmac_body = '%s\n%s\n%s' % (method, expires, path)
@@ -229,6 +268,9 @@ class TestTempURL(unittest.TestCase):
         resp = req.get_response(self.tempurl)
         self.assertEqual(resp.status_int, 200)
         self.assertEqual(resp.headers['content-disposition'], 'inline')
+        self.assertIn('expires', resp.headers)
+        self.assertEqual('Thu, 01 Jan 1970 00:00:01 GMT',
+                         resp.headers['expires'])
         self.assertEqual(req.environ['swift.authorize_override'], True)
         self.assertEqual(req.environ['REMOTE_USER'], '.wsgi.tempurl')
 
@@ -246,6 +288,7 @@ class TestTempURL(unittest.TestCase):
         resp = req.get_response(self.tempurl)
         self.assertEqual(resp.status_int, 200)
         self.assertEqual(resp.headers['content-disposition'], 'inline')
+        self.assertIn('expires', resp.headers)
         self.assertEqual(req.environ['swift.authorize_override'], True)
         self.assertEqual(req.environ['REMOTE_USER'], '.wsgi.tempurl')
 
@@ -265,6 +308,7 @@ class TestTempURL(unittest.TestCase):
         self.assertEqual(resp.headers['content-disposition'],
                          'attachment; filename="a%0D%0Ab"; ' +
                          "filename*=UTF-8''a%0D%0Ab")
+        self.assertIn('expires', resp.headers)
         self.assertEqual(req.environ['swift.authorize_override'], True)
         self.assertEqual(req.environ['REMOTE_USER'], '.wsgi.tempurl')
 
@@ -284,6 +328,7 @@ class TestTempURL(unittest.TestCase):
         self.assertEqual(resp.status_int, 200)
         self.assertEqual(resp.headers['content-disposition'],
                          'attachment; filename="fu%0Abar"')
+        self.assertIn('expires', resp.headers)
         self.assertEqual(req.environ['swift.authorize_override'], True)
         self.assertEqual(req.environ['REMOTE_USER'], '.wsgi.tempurl')
 
@@ -303,6 +348,7 @@ class TestTempURL(unittest.TestCase):
         self.assertEqual(resp.headers['content-disposition'],
                          'attachment; filename="o"; ' +
                          "filename*=UTF-8''o")
+        self.assertIn('expires', resp.headers)
         self.assertEqual(req.environ['swift.authorize_override'], True)
         self.assertEqual(req.environ['REMOTE_USER'], '.wsgi.tempurl')
 
@@ -323,6 +369,7 @@ class TestTempURL(unittest.TestCase):
             resp.headers['content-disposition'],
             'attachment; filename="/i/want/this/just/as/it/is/"; ' +
             "filename*=UTF-8''/i/want/this/just/as/it/is/")
+        self.assertIn('expires', resp.headers)
         self.assertEqual(req.environ['swift.authorize_override'], True)
         self.assertEqual(req.environ['REMOTE_USER'], '.wsgi.tempurl')
 
@@ -339,7 +386,8 @@ class TestTempURL(unittest.TestCase):
                 sig, expires)})
         resp = req.get_response(self.tempurl)
         self.assertEqual(resp.status_int, 404)
-        self.assertFalse('content-disposition' in resp.headers)
+        self.assertNotIn('content-disposition', resp.headers)
+        self.assertNotIn('expires', resp.headers)
         self.assertEqual(req.environ['swift.authorize_override'], True)
         self.assertEqual(req.environ['REMOTE_USER'], '.wsgi.tempurl')
 
