@@ -26,7 +26,8 @@ from swift.common.storage_policy import (
     BaseStoragePolicy, StoragePolicy, ECStoragePolicy, REPL_POLICY, EC_POLICY,
     VALID_EC_TYPES, DEFAULT_EC_OBJECT_SEGMENT_SIZE, BindPortsCache)
 from swift.common.ring import RingData
-from swift.common.exceptions import RingValidationError
+from swift.common.exceptions import RingLoadError
+from pyeclib.ec_iface import ECDriver
 
 
 @BaseStoragePolicy.register('fake')
@@ -374,6 +375,15 @@ class TestStoragePolicies(unittest.TestCase):
         # but only because automated testing requires it.
         policies = parse_storage_policies(name_repeat_conf)
 
+        extra_commas_conf = self._conf("""
+        [storage-policy:0]
+        name = one
+        aliases = ,,one, ,
+        default = yes
+        """)
+        # Extra blank entries should be silently dropped
+        policies = parse_storage_policies(extra_commas_conf)
+
         bad_conf = self._conf("""
         [storage-policy:0]
         name = one
@@ -499,6 +509,9 @@ class TestStoragePolicies(unittest.TestCase):
         self.assertRaisesWithMessage(PolicyError, 'Invalid name',
                                      policies.add_policy_alias, 2, 'double\n')
 
+        self.assertRaisesWithMessage(PolicyError, 'Invalid name',
+                                     policies.add_policy_alias, 2, '')
+
         # try to add existing name
         self.assertRaisesWithMessage(PolicyError, 'Duplicate name',
                                      policies.add_policy_alias, 2, 'two')
@@ -1107,9 +1120,9 @@ class TestStoragePolicies(unittest.TestCase):
 
     def test_quorum_size_replication(self):
         expected_sizes = {1: 1,
-                          2: 2,
+                          2: 1,
                           3: 2,
-                          4: 3,
+                          4: 2,
                           5: 3}
         for n, expected in expected_sizes.items():
             policy = StoragePolicy(0, 'zero',
@@ -1133,23 +1146,32 @@ class TestStoragePolicies(unittest.TestCase):
         test_policies = [
             ECStoragePolicy(0, 'ec8-2', ec_type=DEFAULT_TEST_EC_TYPE,
                             ec_ndata=8, ec_nparity=2,
-                            object_ring=FakeRing(replicas=8),
                             is_default=True),
             ECStoragePolicy(1, 'ec10-4', ec_type=DEFAULT_TEST_EC_TYPE,
-                            ec_ndata=10, ec_nparity=4,
-                            object_ring=FakeRing(replicas=10)),
+                            ec_ndata=10, ec_nparity=4),
             ECStoragePolicy(2, 'ec4-2', ec_type=DEFAULT_TEST_EC_TYPE,
-                            ec_ndata=4, ec_nparity=2,
-                            object_ring=FakeRing(replicas=7)),
+                            ec_ndata=4, ec_nparity=2),
         ]
+        actual_load_ring_replicas = [8, 10, 7]
         policies = StoragePolicyCollection(test_policies)
 
-        for policy in policies:
-            msg = 'EC ring for policy %s needs to be configured with ' \
-                  'exactly %d nodes.' % \
-                  (policy.name, policy.ec_ndata + policy.ec_nparity)
-            self.assertRaisesWithMessage(RingValidationError, msg,
-                                         policy._validate_ring)
+        def create_mock_ring_data(num_replica):
+            class mock_ring_data_klass(object):
+                def __init__(self):
+                    self._replica2part2dev_id = [0] * num_replica
+
+            return mock_ring_data_klass()
+
+        for policy, ring_replicas in zip(policies, actual_load_ring_replicas):
+            with mock.patch('swift.common.ring.ring.RingData.load',
+                            return_value=create_mock_ring_data(ring_replicas)):
+                with mock.patch(
+                        'swift.common.ring.ring.validate_configuration'):
+                    msg = 'EC ring for policy %s needs to be configured with ' \
+                          'exactly %d replicas.' % \
+                          (policy.name, policy.ec_ndata + policy.ec_nparity)
+                    self.assertRaisesWithMessage(RingLoadError, msg,
+                                                 policy.load_ring, 'mock')
 
     def test_storage_policy_get_info(self):
         test_policies = [
@@ -1232,6 +1254,29 @@ class TestStoragePolicies(unittest.TestCase):
             expected_info = expected[(int(policy), False)]
             self.assertEqual(policy.get_info(config=False), expected_info)
 
+    def test_ec_fragment_size_cached(self):
+        policy = ECStoragePolicy(
+            0, 'ec2-1', ec_type=DEFAULT_TEST_EC_TYPE,
+            ec_ndata=2, ec_nparity=1, object_ring=FakeRing(replicas=3),
+            ec_segment_size=DEFAULT_EC_OBJECT_SEGMENT_SIZE, is_default=True)
+
+        ec_driver = ECDriver(ec_type=DEFAULT_TEST_EC_TYPE,
+                             k=2, m=1)
+        expected_fragment_size = ec_driver.get_segment_info(
+            DEFAULT_EC_OBJECT_SEGMENT_SIZE,
+            DEFAULT_EC_OBJECT_SEGMENT_SIZE)['fragment_size']
+
+        with mock.patch.object(
+                policy.pyeclib_driver, 'get_segment_info') as fake:
+            fake.return_value = {
+                'fragment_size': expected_fragment_size}
+
+            for x in range(10):
+                self.assertEqual(expected_fragment_size,
+                                 policy.fragment_size)
+                # pyeclib_driver.get_segment_info is called only once
+                self.assertEqual(1, fake.call_count)
+
 
 if __name__ == '__main__':
     unittest.main()
